﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEngine.UI;
using System;

[AddComponentMenu("UI/Effects/Curved Text")]
[RequireComponent(typeof(Text),typeof(RectTransform))]
#if UNITY_5
public class CurvedText : BaseMeshEffect
#else
public class CurvedText : BaseVertexEffect
#endif
{
	public AnimationCurve curveForText = AnimationCurve.Linear (0, 0, 1, 10);
	public float curveMultiplier = 1;

#if UNITY_EDITOR
		protected override void OnValidate ()
		{
				base.OnValidate ();
				if (curveForText [0].time != 0) {
						var tmpRect = curveForText [0];
						tmpRect.time = 0;
						curveForText.MoveKey (0, tmpRect);
				}
				if (curveForText [curveForText.length - 1].time != GetComponent<RectTransform> ().rect.width)
						OnRectTransformDimensionsChange ();
		}
#endif
	protected override void Awake ()
	{
		base.Awake ();
		OnRectTransformDimensionsChange ();
	}

	protected override void OnEnable ()
	{
		base.OnEnable ();
		OnRectTransformDimensionsChange ();
	}

#if UNITY_5
	public override void ModifyMesh (VertexHelper vh)
	{
		if (IsActive ()) {
			List<UIVertex> verts = new List<UIVertex>();
			vh.GetUIVertexStream (verts);
			
			ApplyCurve (verts);
			
			vh.Clear ();
			vh.AddUIVertexTriangleStream (verts);
		}
	}
#else
	public override void ModifyVertices (List<UIVertex> verts)
	{
		if (IsActive ())
			ApplyCurve (verts);
	}
#endif

	public void ApplyCurve (List<UIVertex> verts)
	{
		for (int index = 0; index < verts.Count; index++) {
			var uiVertex = verts [index];
			RectTransform rectTrans = GetComponent<RectTransform> ();
			uiVertex.position.y += curveForText.Evaluate (rectTrans.rect.width * rectTrans.pivot.x + uiVertex.position.x) * curveMultiplier;
			verts [index] = uiVertex;
		}
	}

	protected override void OnRectTransformDimensionsChange ()
	{
		var tmpRect = curveForText [curveForText.length - 1];
		tmpRect.time = GetComponent<RectTransform> ().rect.width;
		curveForText.MoveKey (curveForText.length - 1, tmpRect);
	}
}
