﻿using UnityEngine;
using UnityEngine.Events;
using System.Collections;

public class Bomb : MonoBehaviour
{
	public Vector3 startVelocity = Vector3.forward * 10;
	public float warningTime = 3;
	public float explodeDelay = 5;
	public float explodeHeight = -3;
	public float explosionForce = 10;
	public float explosionRadius = 30;

	public BombAim bombAimPrefab;

	public UnityEvent onWarning;
	public UnityEvent onExplode;
	public bool showLog = false;
	private float startTime = 0;

	IEnumerator Start ()
	{
		startTime = Time.time;
		if (startVelocity != Vector3.zero)
			GetComponent<Rigidbody> ().velocity = transform.TransformVector (startVelocity);

		BombAim aim = GameObject.Instantiate<BombAim> (bombAimPrefab);
		aim.bomb = this;
		aim.UpdatePositionByBomb ();

		yield return new WaitForSeconds (explodeDelay - warningTime);

		onWarning.Invoke ();

		yield return new WaitForSeconds (warningTime);
		Explode ();
	}

	void FixedUpdate ()
	{
		Vector3 velocity = GetComponent<Rigidbody> ().velocity;

		if (Mathf.Abs (velocity.x) > Mathf.Abs (velocity.z))
			velocity = new Vector3 (velocity.x, velocity.y, 0);
		else
			velocity = new Vector3 (0, velocity.y, velocity.z);

		GetComponent<Rigidbody> ().velocity = velocity;

		if (transform.position.y <= explodeHeight)
			Explode ();
	}

	public void Explode ()
	{
		onExplode.Invoke ();
		this.enabled = false;

		bool win = transform.position.y < 0;
		Messenger.Broadcast (GameMessageTypes.GameEnded, win, MessengerMode.DONT_REQUIRE_LISTENER);
		if (!win)
			Room.Instance.Explode (explosionForce, transform.position, explosionRadius);
	}

	void OnGUI ()
	{
		if (showLog)
			GUI.Box (new Rect (0, 0, 100, 30), string.Format ("{0:0.0}", Time.time - startTime));
	}
}
