﻿using UnityEngine;
using UnityEngine.Events;
using System.Collections;
using System.Collections.Generic;

[RequireComponent (typeof(AudioSource))]
public class Gadget : MouseEventDispatcher
{
	public Material highlightMaterial;
	public AudioClipPlaySetting hoverInSound;
	public AudioClipPlaySetting clickSound;
	private List<GadgetBehavior> gadgetBehaviors;
	private List<MeshRenderer> childMeshRenderers;

	public bool IsHighlighted {
		set {
			foreach (MeshRenderer mr in childMeshRenderers) {
				if (value)
					mr.materials = new Material[]{ mr.material, highlightMaterial };
				else
					mr.materials = new Material[]{ mr.material };
			}
		}
	}

	void Awake ()
	{
		gadgetBehaviors = new List<GadgetBehavior> (GetComponentsInChildren<GadgetBehavior> ());
		childMeshRenderers = new List<MeshRenderer> (GetComponentsInChildren<MeshRenderer> ());
	}

	protected override void OnValidClick ()
	{
		base.OnValidClick ();

		gadgetBehaviors.ForEach (b => b.IsWorking = !b.IsWorking);
		GetComponent<AudioSource> ().PlayOneShot (clickSound);
	}

	protected override void OnHoverIn (InteractionMessagesSender.PointerMessageData pointerMessageData)
	{
		base.OnHoverIn (pointerMessageData);
		IsHighlighted = true;

		GetComponent<AudioSource> ().PlayOneShot (hoverInSound);
	}

	protected override void OnHoverOut (InteractionMessagesSender.PointerMessageData pointerMessageData)
	{
		base.OnHoverOut (pointerMessageData);
		IsHighlighted = false;
	}

	public void StopWorking ()
	{
		gadgetBehaviors.ForEach (b => b.IsWorking = false);
		IsHighlighted = false;
	}
}
