﻿using UnityEngine;
using System.Collections;

public abstract class GadgetBehaviorTimed : GadgetBehavior
{
	public delegate void GenericEventHandler (GadgetBehaviorTimed behavior);

	public event GenericEventHandler TweenStarted;
	public event GenericEventHandler TweenCompeted;

	[UnityEngine.Serialization.FormerlySerializedAs ("openTime")]
	public float time = 1;
	public iTween.EaseType easeType = iTween.EaseType.linear;
	private Tweener workingProgressTweener;

	public bool IsTweening {
		get { return !workingProgressTweener.IsCompleted; }
	}

	protected override void OnIsWorkingChanged ()
	{
		if (IsWorking)
			StartCoroutine ("TweenWorkingProgressTo", 1);
		else
			StartCoroutine ("TweenWorkingProgressTo", 0);
	}

	void Start ()
	{
		workingProgressTweener = new Tweener (0, 0, time, easeType);
		workingProgressTweener.TimeElapsed = time;
	}

	private IEnumerator TweenWorkingProgressTo (float progress)
	{
		if (TweenStarted != null)
			TweenStarted (this);
		
		workingProgressTweener.from = workingProgressTweener.Value;
		workingProgressTweener.to = progress;
		workingProgressTweener.time = Mathf.Abs (progress - workingProgressTweener.from) * time;

		workingProgressTweener.TimeElapsed = 0;
		while (!workingProgressTweener.IsCompleted) {
			if (GetComponent<Rigidbody> () != null) {
				yield return new WaitForFixedUpdate ();
				workingProgressTweener.TimeElapsed += Time.fixedDeltaTime;
			} else {
				yield return null;
				workingProgressTweener.TimeElapsed += Time.deltaTime;
			}

			OnWorkingProgressChanged (workingProgressTweener.Value);
		}

		if (TweenCompeted != null)
			TweenCompeted (this);
	}

	protected abstract void OnWorkingProgressChanged (float currentProgress);
}
