﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Room : MonoBehaviour
{
	private static ObjectInstanceFinder<Room> instanceFinder = new ObjectInstanceFinder<Room> ();

	public static Room Instance{ get { return instanceFinder.Instance; } }

	private List<Gadget> gadgets;
	private List<ExplosionPiece> explosionPieces;

	public bool Interactable {
		get { return InteractionMessagesSender.Current.gameObject.activeSelf; }
		set { InteractionMessagesSender.Current.gameObject.SetActive (value); }
	}

	public List<Gadget> Gadgets {
		get {
			if (gadgets == null) {
				gadgets = new List<Gadget> (GetComponentsInChildren<Gadget> (true));
			}
			return gadgets;
		}
	}

	public List<ExplosionPiece> ExplosionPieces {
		get {
			if (explosionPieces == null) {
				explosionPieces = new List<ExplosionPiece> (GetComponentsInChildren<ExplosionPiece> ());
			}
			return explosionPieces;
		}
	}

	public void Reset ()
	{
		Gadgets.ForEach (g => g.StopWorking ());
		ExplosionPieces.ForEach (e => e.Reset ());
	}

	public void Explode (float force, Vector3 position, float radius)
	{
		ExplosionPieces.ForEach (e => e.Explode (force, position, radius));
	}
}
