﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class GameSceneStateGame : GameSceneState
{
	public Transform bombsRoot;

	private List<Bomb> bombs = new List<Bomb> ();
	private int currentBombIndex = 0;

	public override GameSceneStateTypes StateId { get { return GameSceneStateTypes.Game; } }

	public bool HasMoreBombs{ get { return currentBombIndex < bombs.Count; } }

	void Awake ()
	{
		bombs = new List<Bomb> (bombsRoot.GetComponentsInChildren<Bomb> (true));
		if (GameSceneController.Instance.testGameLevels) {
			List<Bomb> testBombs = new List<Bomb> ();
			foreach (Bomb bomb in bombs) {
				if (bomb.gameObject.activeSelf)
					testBombs.Add (bomb);
			}
			bombs = testBombs;
		}
	}

	public override void BeginState (GameSceneStateTypes previousState)
	{
		base.BeginState (previousState);

		StartCoroutine ("PrepareAndStartGame");
		Messenger.AddListener (GameMessageTypes.GameEnded, OnGameEnded);
	}

	public override void EndState (GameSceneStateTypes newState)
	{
		base.EndState (newState);

		StopCoroutine ("PrepareAndStartGame");

		Messenger.RemoveListener (GameMessageTypes.GameEnded, OnGameEnded);
		Room.Instance.Interactable = false;
		Room.Instance.Reset ();
	}

	private IEnumerator PrepareAndStartGame ()
	{
		Room.Instance.Reset ();

		yield return new WaitForSeconds (1);

		currentBombIndex = Mathf.Clamp (currentBombIndex, 0, bombs.Count - 1);
		Bomb bombPrefab = bombs [currentBombIndex];

		Bomb bomb = GameObject.Instantiate<Bomb> (bombPrefab);
		bomb.gameObject.SetActive (true);
		bomb.enabled = true;

		Room.Instance.Interactable = true;
	}

	private void OnGameEnded (object data)
	{
		bool win = (bool)data;
		if (win) {
			currentBombIndex++;
			Owner.FiniteStateMachine.GotoState (GameSceneStateTypes.Win);
		} else
			Owner.FiniteStateMachine.GotoState (GameSceneStateTypes.Lose);
	}

	public void Retry ()
	{
		Bomb[] bombs = GameObject.FindObjectsOfType<Bomb> ();
		foreach (Bomb bomb in bombs)
			Destroy (bomb.gameObject);

		StopCoroutine ("PrepareAndStartGame");
		StartCoroutine ("PrepareAndStartGame");
	}

	public void Reset ()
	{
		currentBombIndex = 0;
	}
}
